@@ -31,11 +31,11 @@ extern void g_none(char *first, ...);
 #endif
  /**/
 
-#define CURRENT_BFLANG2_VERSION "2.0"
-
 #define BF_OFFSET_UNDEFINED G_MAXINT32
 #define BF_POSITION_UNDEFINED G_MAXINT32
 
+/*#define DUMP_CONTEXTS*/
+/*#define DUMP_PATTERNS*/
 /*#define DUMP_SCANCACHE*/
 /*#define DUMP_SCANCACHE_UPDATE_OFFSET*/
 /*#define CHECK_CONSISTENCY*/
@@ -81,7 +81,9 @@ typedef struct {
 typedef struct {
 	GArray *table; /* a pointer to the DFA table for this context */
 	GCompletion *ac;			/* autocompletion items in this context */
-	GHashTable *patternhash;	/* a hash table where the pattern and its autocompletion string are the keys, and an integer to the ID of the pattern is the value */
+	GHashTable *patternhash;	/* a hash table where the pattern and its autocompletion	strings are the keys, and an integer
+										to the ID of the pattern is the value. Only patterns that have backup cursor, a condition, or
+										a reference set are required in this hash. */
 	GtkTextTag *contexttag;		/* if the context area itself needs some kind of style (to implement a string context for example) */
 	gchar *contexthighlight;	/* the string that has the id for the highlight */
 	guint8 has_tagclose_from_blockstack;	/* this context has xml end patterns that need autoclosing for generix xml tags, based on the tag that is on top of the blockstack */
@@ -90,20 +92,15 @@ typedef struct {
 								   or COMMENT_INDEX_NONE if there is no block comment  */
 	guint8 comment_line;		/* index in array scantable->comments for line comments; see comment_block */
 	guint8 autocomplete_case_insens;
+	guint8 autocomplete_has_conditions; /* 0 means any autocomplete item is valid, 1 means we have to check for runtime conditions */
 	guint8 default_spellcheck;
 	guint8 dump_dfa_run;
 } Tcontext;
 /*
-32bit size = 5*32 + 6*8 = 200 + 16 padding bits = 28 bytes
-64bit size = 5*64 + 6*8 = 360 + 16 padding bits = 48 bytes
+32bit size = 5*32 + 7*8 = 208 + 8 padding bits = 28 bytes
+64bit size = 5*64 + 7*8 = 368 + 8 padding bits = 48 bytes
 */
 
-typedef struct {
-	gchar *autocomplete_string;
-	guint8 autocomplete_backup_cursor;	/* number of characters to backup the cursor after autocompletion (max 256) */
-} Tpattern_autocomplete;
-
-#ifdef CONDITIONALPATTERN
 typedef struct {
 	gchar *refname; 	/* because we can match the name only after all contexts are loaded, we store it in the structure */
 	guint16 ref; 			/* if the reference was a context, the ID of that context, if the reference
@@ -115,7 +112,19 @@ typedef struct {
 									4 = invalid if relation with block matches
 									*/
 } Tpattern_condition;
-#endif /* CONDITIONALPATTERN */
+
+typedef struct {
+	gchar *autocomplete_string;
+	guint16 condition; /* 0 for most patterns, only blocks that are (in)valid in a certain condition have this set (refers to a Tpattern_condition) */
+	guint8 autocomplete_backup_cursor;	/* number of characters to backup the cursor after autocompletion (max 256) */
+	guint8 trigger_new_autocomp_popup; /* if a new autocompletion popup should be triggered immediately after the text of 
+													this one is inserted. Useful for example to autocomplete the values of an attribute 
+													after an attribute has been inserted. */
+} Tpattern_autocomplete;
+/*
+32bit size = 32 + 16 + 2*8 = 64 + 0 padding bits = 8 bytes
+64bit size = 64 + 16 + 2*8 = 96 + 32 padding bits = 16 bytes
+*/
 
 typedef struct {
 	gchar *name;
@@ -134,9 +143,7 @@ typedef struct {
 										to be able to recompile a pattern in multiple contexts we need this information in Tpattern */
 	gchar *selfhighlight;		/* a string with the highlight for this pattern. used when re-linking highlights and textstyles
 								   if the user changed any of these in the preferences */
-#ifdef CONDITIONALPATTERN
 	guint16 condition; /* 0 for most patterns, only blocks that are (in)valid in a certain condition have this set */
-#endif
 	guint16 block;			/* this is 0 for most blocks, only blocks that need a tag have this set, refers to a a position in an array of Tpattern_block*/
 	gint16 blockstartpattern;	/* the number of the pattern that may start this block, or -1 to end the last started block, also used for stretch block */
 	gint16 nextcontext;			/* 0, or if this pattern starts a new context the number of the context, or -1 or -2 etc.
