@@ -67,6 +67,52 @@ bfwin_fullscreen_toggle(Tbfwin * bfwin, gboolean active)
 	}
 }
 
+typedef enum {
+	order_byfullpath,
+	order_byfilename
+}Tordering_mode;
+
+static gint
+sort_doc_by_filename(gconstpointer a,gconstpointer b)
+{
+	gint retval=0;
+	if (!a || !b)
+		return 0;
+	return g_strcmp0(gtk_label_get_text(GTK_LABEL(DOCUMENT(a)->tab_label)) , gtk_label_get_text(GTK_LABEL(DOCUMENT(b)->tab_label)));
+}
+
+static gint
+sort_doc_by_fullpath(gconstpointer a,gconstpointer b)
+{
+	if (!a || !b)
+		return 0;
+	return g_strcmp0(gtk_label_get_text(GTK_LABEL(DOCUMENT(a)->tab_menu)) , gtk_label_get_text(GTK_LABEL(DOCUMENT(b)->tab_menu)));
+}
+
+
+static void
+notebooks_reorder(Tbfwin *bfwin, Tordering_mode mode)
+{
+	GList *tmplist, *copylist;
+	gint i=0;
+	copylist = g_list_copy(bfwin->documentlist);
+	switch(mode) {
+		case order_byfullpath:
+			copylist = g_list_sort(copylist, sort_doc_by_fullpath);
+		break;
+		case order_byfilename:
+			copylist = g_list_sort(copylist, sort_doc_by_filename);
+		break;
+	}
+	/* now sort according to the order in the copylist */
+	for (tmplist=g_list_first(copylist);tmplist;tmplist=tmplist->next) {
+		DEBUG_MSG("notebooks_reorder, move document %s to position %d\n",gtk_label_get_text(GTK_LABEL(DOCUMENT(tmplist->data)->tab_menu)), i);
+		gtk_notebook_reorder_child(GTK_NOTEBOOK(bfwin->notebook), DOCUMENT(tmplist->data)->vsplit,i);
+		i++;
+	}
+	g_list_free(copylist);
+}
+
 static void
 notebook_move(Tbfwin * bfwin, gboolean move_left)
 {
@@ -119,6 +165,12 @@ bfwin_notebook_switch(Tbfwin * bfwin, guint action)
 	case 7:
 		g_print("bfwin_notebook_switch, should not be called for 'recent'\n");
 		break;
+	case 8:
+		notebooks_reorder(bfwin, order_byfilename);
+		break;
+	case 9:
+		notebooks_reorder(bfwin, order_byfullpath);
+		break;
 	}
 }
 
@@ -837,7 +889,7 @@ bfwin_gotoline_search_bar_close(Tbfwin *bfwin, gboolean clean_entry_only)
 
 		if (bfwin->simplesearch_snr3run) {
 			DEBUG_MSG("free simple search run %p\n", bfwin->simplesearch_snr3run);
-			snr3run_free(bfwin->simplesearch_snr3run, TRUE);
+			snr3run_free(bfwin->simplesearch_snr3run, TRUE, FALSE);
 			bfwin->simplesearch_snr3run = NULL;
 		}
 	}
@@ -884,7 +936,7 @@ simplesearch_start(Tbfwin *bfwin, gboolean allow_single_char_search) {
 	tmpstr = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(bfwin->simplesearch_combo))));
 	if (bfwin->simplesearch_snr3run) {
 		DEBUG_MSG("free simple search run %p\n", bfwin->simplesearch_snr3run);
-		snr3run_free(bfwin->simplesearch_snr3run, TRUE);
+		snr3run_free(bfwin->simplesearch_snr3run, TRUE, FALSE);
 		bfwin->simplesearch_snr3run=NULL;
 	}
 	if (tmpstr && tmpstr[0]!='\0' && (allow_single_char_search || tmpstr[1] != '\0')) {
@@ -1516,7 +1568,7 @@ bfwin_create_main(Tbfwin * bfwin)
 
 	bftextview2_identifier_hash_init(bfwin);
 	bfwin->main_window =
-		window_full2(_("New Bluefish Window"), GTK_WIN_POS_CENTER, 0, G_CALLBACK(bfwin_destroy_event),
+		window_full2(_("New Bluefish Window"), 0, 0, G_CALLBACK(bfwin_destroy_event),
 					 bfwin, FALSE, NULL);
 	gtk_window_set_role(GTK_WINDOW(bfwin->main_window), "bluefish");
 /*#ifndef MAC_INTEGRATION
