@@ -2,7 +2,7 @@
  * Bluefish HTML Editor
  * bfwin_uimanager.c
  *
- * Copyright (C) 2011, 2015 James Hayward and Olivier Sessink
+ * Copyright (C) 2011, 2015, 2017 James Hayward and Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -141,6 +141,17 @@ ui_move_tab_right(GtkAction * action, gpointer user_data)
 	bfwin_notebook_switch(BFWIN(user_data), 6);
 }
 
+static void
+ui_reorder_tab_byfilename(GtkAction * action, gpointer user_data)
+{
+	bfwin_notebook_switch(BFWIN(user_data), 8);
+}
+static void
+ui_reorder_tab_byfullpath(GtkAction * action, gpointer user_data)
+{
+	bfwin_notebook_switch(BFWIN(user_data), 9);
+}
+
 static void
 ui_set_autoindent(GtkAction * action, gpointer user_data)
 {
@@ -674,6 +685,12 @@ ui_select_block(GtkAction * action, gpointer user_data)
 	select_between_matching_block_boundaries(BFWIN(user_data)->current_document);
 }
 
+static void
+ui_select_identifier(GtkAction * action, gpointer user_data)
+{
+	select_current_identifier(BFWIN(user_data)->current_document);
+}
+
 static void
 ui_duplicate_line(GtkAction * action, gpointer user_data)
 {
@@ -826,6 +843,7 @@ static const GtkActionEntry top_level_menus[] = {
 	{"DocumentFontSize", NULL, N_("_Font Size")},
 	{"DocumentLangMode", NULL, N_("Language M_ode")},
 	{"DocumentEncoding", NULL, N_("Character _Encoding")},
+	{"DocReorder", NULL, N_("Reorder documents")},
 	{"GoMenu", NULL, N_("_Go")},
 	{"ProjectMenu", NULL, N_("_Project")},
 	{"ProjectOpenRecent", NULL, N_("Open _Recent")},
@@ -887,6 +905,8 @@ static const GtkActionEntry global_actions[] = {
 	{"SelectBlock", NULL, N_("Select _Block"), "<shift><control>b",
 	 N_("Select block, use multiple times to select parent blocks "),
 	 G_CALLBACK(ui_select_block)},
+	{"SelectIdentifier", NULL, N_("Select _Identifier"), "<shift><control>i",
+	 N_("Select identifier"), G_CALLBACK(ui_select_identifier)},
 	{"DuplicateLine", NULL, N_("D_uplicate Line"), "<control>d", N_("Duplicate the current line"),
 	 G_CALLBACK(ui_duplicate_line)},
 	{"DeleteLine", NULL, N_("_Delete Line"), "<control>y", N_("Delete the current line"),
@@ -978,6 +998,10 @@ static const GtkActionEntry document_actions[] = {
 	 G_CALLBACK(ui_move_tab_left)},
 	{"DocMoveRight", NULL, N_("Move Tab _Right"), NULL, N_("Move current tab right"),
 	 G_CALLBACK(ui_move_tab_right)},
+	 {"DocReorderByfilename", NULL, N_("Sort by filename"), NULL, N_("Sort by filename"),
+	 G_CALLBACK(ui_reorder_tab_byfilename)},
+	 {"DocReorderByfullpath", NULL, N_("Sort by full path"), NULL, N_("Sort by full path"),
+	 G_CALLBACK(ui_reorder_tab_byfullpath)},
 	{"DocRecent", NULL, N_("_Recent Document"), "<control>Tab", N_("Goto most recent document"),
 	 G_CALLBACK(ui_doc_recent)},
 	{"DocFirst", GTK_STOCK_GOTO_FIRST, N_("_First Document"), "<shift><control>Page_Up",
@@ -1618,7 +1642,7 @@ bfwin_commands_menu_create(Tbfwin * bfwin)
 		dynamic_menu_empty(bfwin->uimanager, bfwin->commands_merge_id, bfwin->commands_group);
 	}
 	bfwin->commands_merge_id = gtk_ui_manager_new_merge_id(bfwin->uimanager);
-	for (list = g_list_first(main_v->props.external_command); list; list = list->next) {
+	for (list = g_list_last(main_v->props.external_command); list; list = list->prev) {
 		gchar **arr = list->data;
 		/*  arr[0] = state
 		 *  arr[1] = name
@@ -1787,7 +1811,7 @@ bfwin_filters_menu_create(Tbfwin * bfwin)
 
 	bfwin->filters_merge_id = gtk_ui_manager_new_merge_id(bfwin->uimanager);
 
-	for (list = g_list_first(main_v->props.external_filter); list; list = list->next) {
+	for (list = g_list_last(main_v->props.external_filter); list; list = list->prev) {
 		gchar **arr = list->data;
 		/* arr[0] = state
 		 * arr[1] = name
@@ -1827,7 +1851,7 @@ bfwin_outputbox_menu_create(Tbfwin * bfwin)
 
 	bfwin->outputbox_merge_id = gtk_ui_manager_new_merge_id(bfwin->uimanager);
 
-	for (list = g_list_first(main_v->props.external_outputbox); list; list = list->next) {
+	for (list = g_list_last(main_v->props.external_outputbox); list; list = list->prev) {
 		gchar **arr = list->data;
 		/* arr[0] = state
 		 * arr[1] = name
@@ -1871,6 +1895,7 @@ recent_menu_remove_backend(Tbfwin * bfwin, const gchar * menupath, const gchar *
 	menuitem = gtk_ui_manager_get_widget(bfwin->uimanager, menupath);
 	menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menuitem));
 	list = gtk_container_get_children(GTK_CONTAINER(menu));
+	DEBUG_MSG("recent_menu_remove_backend, remove item with label %s from menupath %s\n",label,menupath);
 	for (tmplist = list; tmplist; tmplist = tmplist->next) {
 		if (g_strcmp0(gtk_menu_item_get_label(tmplist->data), label) == 0) {
 			gtk_widget_destroy(tmplist->data);
@@ -1885,17 +1910,17 @@ bfwin_recent_menu_remove(Tbfwin * bfwin, gboolean project, const gchar * curi, G
 {
 	GList *tmplist;
 	gchar *label;
-	
+	DEBUG_MSG("bfwin_recent_menu_remove curi=%s\n",curi);
 	label = recent_menu_label(curi, uri);
 	if (!project && bfwin->session != main_v->session) {
-		recent_menu_remove_backend(bfwin, "/MainMenu/FileMenu/FileOpenRecent", curi);
+		recent_menu_remove_backend(bfwin, "/MainMenu/FileMenu/FileOpenRecent", label);
 		return;
 	}
 	for (tmplist = g_list_first(main_v->bfwinlist); tmplist; tmplist = g_list_next(tmplist)) {
 		if (project || BFWIN(tmplist->data)->session == main_v->session) {
 			recent_menu_remove_backend(bfwin,
 									   project ? "/MainMenu/ProjectMenu/ProjectOpenRecent" :
-									   "/MainMenu/FileMenu/FileOpenRecent", curi);
+									   "/MainMenu/FileMenu/FileOpenRecent", label);
 		}
 	}
 	if (label != curi) {
