@@ -104,6 +104,7 @@ static void init_default_session(Tsessionvars *session) {
 			= session->view_blockstack
 			= session->autocomplete
 			= session->show_mbhl
+			= session->snr3_ignorebackupfiles
 			= 1;
 #ifdef HAVE_LIBENCHANT
 	session->spell_check_default = 1;
@@ -128,6 +129,11 @@ static void handle_signals(void) {
 }
 #endif
 
+#ifdef WINLOGFILE
+#include <glib/gstdio.h>
+FILE *winlogfile = NULL;
+#endif
+
 #ifdef MAC_INTEGRATION
 static gboolean osx_open_file_cb(GtkosxApplication *app, gchar *path, gpointer user_data) {
 	GList *tmplist;
@@ -307,6 +313,29 @@ int main(int argc, char *argv[])
 	GPollFunc gdk_poll_func;
 #endif
 
+#ifdef WINLOGFILE
+	// Create logfile
+	gchar *winlogpath = g_strconcat(g_get_home_dir(), "/."PACKAGE"/bluefish.log", NULL);
+    winlogfile = g_fopen(winlogpath, "w");
+	g_free(winlogpath);
+
+	void LogPrint(const gchar *message) {
+		g_fprintf(winlogfile, "%s", message);
+		fflush(winlogfile);
+	}
+
+	void LogDebug(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data) {
+		g_fprintf(winlogfile, "%s", message);
+		fflush(winlogfile);
+	}
+
+	// If the logfile was created successfully, redirect output to the log
+	if (winlogfile) {
+		g_set_print_handler(LogPrint);
+		g_log_set_handler(NULL, G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION, LogDebug, NULL);
+	}
+#endif
+
 #ifndef WIN32
 	/* the unity appmenu cannot handle an application with two menu's
 	for now I didn't find a way to make it honor the main menu and ignore the snippets menu, so
@@ -559,6 +588,14 @@ int main(int argc, char *argv[])
 #else
 	exit(0);
 #endif
+
+#ifdef WINLOGFILE
+	// Close logfile
+	if (winlogfile) {
+		fclose(winlogfile);
+	}
+#endif
+
 	return 0;
 }
 
