@@ -1118,8 +1118,7 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	GtkWidget *vbox, *hbox, *but;
 	GdkColor gcolor;
 	const gchar *this_color=setcolor;
-
-	csd = g_malloc(sizeof(Tcolsel));
+	csd = g_new0(Tcolsel, 1);
 	/* warning: bfwin might be NULL if this dialog is started by return_color() */
 	csd->bfwin = bfwin;
 	csd->is_modal = modal;
@@ -1128,7 +1127,7 @@ static Tcolsel *colsel_dialog(Tbfwin *bfwin,const gchar *setcolor, gint modal, g
 	csd->returnval = setcolor ? g_strdup(setcolor) : g_strdup("");
 	
 	DEBUG_MSG("colsel_dialog, malloced at %p, setcolor=%s\n", csd, setcolor);
-	csd->win = window_full2(_("Bluefish: Select color"), GTK_WIN_POS_MOUSE, 12, G_CALLBACK(colsel_destroy_lcb), csd, TRUE, NULL);
+	csd->win = window_full2(_("Bluefish: Select color"), GTK_WIN_POS_CENTER, 12, G_CALLBACK(colsel_destroy_lcb), csd, TRUE, bfwin?bfwin->main_window:NULL);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(csd->win), vbox);
 	csd->csel = gtk_color_selection_new();
