@@ -110,7 +110,7 @@ project_setup_initial_session(Tsessionvars * session, gboolean before_parse)
 
 	/* THE NUBER OF BYTES THAT IS COPIED HERE MUST BE EQUAL TO THE NUMBER OF INTEGERS
 	   IN THE STRUCT IN bluefish.h ELSE ALL WILL FAIL */
-	memcpy(session, main_v->session, 45 * sizeof(gint));
+	memcpy(session, main_v->session, 47 * sizeof(gint));
 }
 
 Tbfwin *
@@ -282,6 +282,18 @@ create_new_project(Tbfwin * bfwin)
 	return prj;
 }
 
+static void
+set_last_project_dir_from_file_uri(GFile *file)
+{
+	GFile *dir = g_file_get_parent(file);
+	if (dir) {
+		g_free(main_v->globses.last_project_dir);
+		main_v->globses.last_project_dir = g_file_get_uri(dir);
+		g_object_unref(dir);
+	}
+}
+
+
 gboolean
 project_save(Tbfwin * bfwin, gboolean save_as)
 {
@@ -313,6 +325,7 @@ project_save(Tbfwin * bfwin, gboolean save_as)
 		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 			gchar *filename;
 			newuri = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
+			set_last_project_dir_from_file_uri(newuri);
 			filename = g_file_get_parse_name(newuri);
 			suflen = strlen(main_v->props.project_suffix);
 			filen = strlen(filename);
@@ -440,11 +453,11 @@ project_open_from_file(Tbfwin * bfwin, GFile * fromuri)
 				uri = g_file_new_for_uri(tmparr[0]);
 			}*/
 
-			if (tmparr[1] && tmparr[1]!='\0') {
+			if (tmparr[1] && tmparr[1]!=NULL) {
 				cursor_offset = atoi(tmparr[1]);
-				if (tmparr[2] && tmparr[2]!='\0') {
+				if (tmparr[2] && tmparr[2]!=NULL) {
 					goto_offset = atoi(tmparr[2]);
-					if (tmparr[3] && tmparr[3]!='\0') {
+					if (tmparr[3] && tmparr[3]!=NULL) {
 						is_active = atoi(tmparr[3]);
 					}
 				}
@@ -478,9 +491,10 @@ static void
 project_open_response_lcb(GtkDialog * dialog, gint response, Tbfwin * bfwin)
 {
 	if (response == GTK_RESPONSE_ACCEPT) {
-		GFile *file;
+		GFile *file, *dir;
 		file = gtk_file_chooser_get_file(GTK_FILE_CHOOSER(dialog));
 		project_open_from_file(bfwin, file);
+		set_last_project_dir_from_file_uri(file);
 		g_object_unref(file);
 	}
 	gtk_widget_destroy(GTK_WIDGET(dialog));
@@ -493,6 +507,8 @@ project_open(Tbfwin * bfwin)
 	dialog =
 		file_chooser_dialog(bfwin, _("Select Bluefish project filename"), GTK_FILE_CHOOSER_ACTION_OPEN, NULL,
 							TRUE, FALSE, "bfproject", FALSE);
+	gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(dialog), main_v->globses.last_project_dir);
+	
 	g_signal_connect(dialog, "response", G_CALLBACK(project_open_response_lcb), bfwin);
 	gtk_widget_show_all(dialog);
 }
@@ -593,7 +609,6 @@ project_edit_destroy_lcb(GtkWidget * widget, Tprojecteditor * pred)
 static void
 project_edit_cancel_clicked_lcb(GtkWidget * widget, Tprojecteditor * pred)
 {
-
 	gtk_widget_destroy(pred->win);
 }
 
@@ -663,7 +678,7 @@ project_edit(Tbfwin * bfwin)
 	pred->bfwin = bfwin;
 	pred->project->editor = pred;
 	pred->win =
-		window_full2(wintitle, GTK_WIN_POS_CENTER, 5, G_CALLBACK(project_edit_destroy_lcb), pred, TRUE,
+		window_full2(wintitle, 0, 5, G_CALLBACK(project_edit_destroy_lcb), pred, TRUE,
 					parent_window);
 	vbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(pred->win), vbox);
@@ -701,7 +716,7 @@ project_edit(Tbfwin * bfwin)
 	gtk_table_attach_defaults(GTK_TABLE(table), pred->entries[template], 2, 3, 3, 4);
 	gtk_table_attach_defaults(GTK_TABLE(table), but, 3, 4, 3, 4);*/
 
-	sessionprefs(_("<b>Project Defaults</b>"), &pred->sprefs, pred->project->session);
+	sessionprefs(_("<b>Default settings for this project</b>"), &pred->sprefs, pred->project->session);
 	gtk_box_pack_start(GTK_BOX(vbox), pred->sprefs.frame, FALSE, FALSE, 6);
 
 #if GTK_CHECK_VERSION(3,0,0)
