@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2014 Olivier Sessink
+ * Copyright (C) 2000-2017 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -275,7 +275,7 @@ save_config_file(GHashTable * config_list, GFile * file)
 			break;
 		case 'a':
 			DEBUG_MSG("save_config_file, tmpitem(%p), &tmpitem=%p\n", tmpitem, &tmpitem);
-			tmplist2 = g_list_last((GList *) * (void **) tmpitem->pointer);
+			tmplist2 = g_list_first((GList *) * (void **) tmpitem->pointer);
 			DEBUG_MSG("save_config_file, the tmplist2(%p)\n", tmplist2);
 			while (tmplist2 != NULL) {
 				tmpstring2 = array_to_string((char **) tmplist2->data);
@@ -285,7 +285,7 @@ save_config_file(GHashTable * config_list, GFile * file)
 					tmpstring = g_strdup_printf("%s", tmpstring2);
 				DEBUG_MSG("save_config_file, tmpstring(%p)=%s\n", tmpstring, tmpstring);
 				rclist = g_list_prepend(rclist, tmpstring);
-				tmplist2 = g_list_previous(tmplist2);
+				tmplist2 = g_list_next(tmplist2);
 				g_free(tmpstring2);
 			}
 			break;
@@ -420,6 +420,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.editor_tab_indent_sel, "editor_tab_indent_sel:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_auto_close_brackets, "editor_auto_close_brackets:", 2, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_replace_unicode_quotes, "editor_replace_unicode_quotes:", 0, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.editor_spacingtoclick, "editor_spacingtoclick:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.use_system_tab_font, "use_system_tab_font:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_shown_filename_len, "max_shown_filename_len:", 0, TRUE);
 	init_prop_string(&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
@@ -811,13 +812,12 @@ GList *update_filters(GList *current, gboolean overwrite)
 #endif
 	defaults =
 		g_list_append(defaults,
-					  array_from_arglist(_("CSS tidy"), "|csstidy --preserve_css=true -|", NULL));
+					  array_from_arglist(_("CSS tidy"), "|csstidy - --preserve_css=true --silent=true |", NULL));
 	retlist = update_externals(current, defaults, overwrite, 3, 2);
 	free_arraylist(defaults);
 	return retlist;
 }
 
-#ifdef WIN32
 static GList *
 add_browser_if_exists(GList *list, const gchar *name, const gchar *path, const gchar *isdefault)
 {
@@ -825,14 +825,17 @@ add_browser_if_exists(GList *list, const gchar *name, const gchar *path, const g
 	
 	uri = g_file_new_for_path(path);
 	if (g_file_query_exists(uri,NULL)) {
+#ifdef WIN32
 		gchar *command = g_strconcat("\"", path, "\" '%p'", NULL);
+#else
+		gchar *command = g_strconcat(path, " '%p'&", NULL);
+#endif
 		list = g_list_prepend(list, array_from_arglist(name,command,isdefault, NULL));
 		g_free(command);
 	}
 	g_object_unref(uri);
 	return list;
 }
-#endif
 
 GList *update_commands(GList *current, gboolean overwrite)
 {
@@ -840,22 +843,8 @@ GList *update_commands(GList *current, gboolean overwrite)
 #ifdef WIN32
 	defaults = add_browser_if_exists(defaults, _("Windows 7 Firefox 32-bit"), "C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe", "1");
 	defaults = add_browser_if_exists(defaults, _("Windows 7 Firefox"), "C:\\Program Files\\Mozilla Firefox\\firefox.exe", "1");
-	
-	defaults =
-		g_list_prepend(defaults,
-					  array_from_arglist(_("Windows 7 Firefox 32-bit"),
-										 "\"C:\\Program Files (x86)\\Mozilla Firefox\\firefox.exe\" '%p'",
-										 "1", NULL));
-	defaults =
-		g_list_prepend(defaults,
-					  array_from_arglist(_("Windows 7 Internet Explorer 32-bit"),
-										 "\"C:\\Program Files (x86)\\Internet Explorer\\iexplore.exe\" '%p'",
-										 "0", NULL));
-	defaults =
-		g_list_prepend(defaults,
-					  array_from_arglist(_("Windows 7 Internet Explorer 64-bit"),
-										 "\"C:\\Program Files\\Internet Explorer\\iexplore.exe\" '%p'",
-										 "0", NULL));
+	defaults = add_browser_if_exists(defaults, _("Windows 7 Internet Explorer 32-bit"), "C:\\Program Files (x86)\\Internet Explorer\\iexplore.exe", "0");
+	defaults = add_browser_if_exists(defaults, _("Windows 7 Internet Explorer 64-bit"), "C:\\Program Files\\Internet Explorer\\iexplore.exe", "0");
 #elif PLATFORM_DARWIN
 	defaults =
 		g_list_prepend(defaults,
@@ -875,32 +864,35 @@ GList *update_commands(GList *current, gboolean overwrite)
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Insert date"), "date|", "0", NULL));
 #else
+	defaults = add_browser_if_exists(defaults, _("Default browser"), "/usr/bin/x-www-browser", "1");
 	defaults =
 		g_list_prepend(defaults,
-					  array_from_arglist(_("Firefox"), "firefox '%p'&", "1", NULL));
-	defaults =
-		g_list_prepend(defaults,
-					  array_from_arglist(_("Opera"), "opera -remote 'openURL(%p)' || opera '%p'&", "0",
-										 NULL));
+					  array_from_arglist(_("Firefox"), "firefox '%p'&", defaults?"0":"1", NULL));
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Chromium"), "chromium '%p' || chromium-browser '%p' &", "0", NULL));
 	defaults =
 		g_list_prepend(defaults,
-					  array_from_arglist(_("Konqueror"), "konqueror '%p'&", "0", NULL));
+					  array_from_arglist(_("Opera"), "opera -remote 'openURL(%p)' || opera '%p'&", "0",
+										 NULL));
+	defaults = add_browser_if_exists(defaults, _("Konqueror"), "/usr/bin/konqueror", "0");
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Epiphany"), "epiphany-browser -n '%p'&", "0", NULL));
-	defaults =
-		g_list_prepend(defaults,
-					  array_from_arglist(_("Kazehakase"), "kazehakase '%p'&", "0", NULL));
+	defaults = add_browser_if_exists(defaults, _("Kazehakase"), "/usr/bin/kazehakase", "0");
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Links2 (graphics)"), "links2 -g '%p'&", "0", NULL));
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("chmod a+x"), "chmod a+x %f", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Insert date"), "date|", "0", NULL));
 #endif
 	retlist = update_externals(current, defaults, overwrite, 4, 2);
 	free_arraylist(defaults);
@@ -937,7 +929,7 @@ rcfile_parse_main(void)
 
 	/* do some default configuration for the lists */
 	DEBUG_MSG("config_version=%s\n",main_v->props.config_version);
-	if (main_v->props.config_version && (strlen(main_v->props.config_version)<5 ||main_v->props.config_version[0] < '2' || main_v->props.config_version[2] < '2' || main_v->props.config_version[4] < '4')) {
+	if (main_v->props.config_version && (strlen(main_v->props.config_version)<5 ||main_v->props.config_version[0] < '2' || main_v->props.config_version[2] < '2' || (main_v->props.config_version[4] < '4' && main_v->props.config_version[5] == '\0') )) {
 		main_v->props.rcfile_from_old_version = 1; /* We got old version after migration */
 	}
 	if (!main_v->props.config_version || (main_v->props.rcfile_from_old_version == 1)) {
@@ -1159,6 +1151,7 @@ rcfile_load_accelerators(gboolean defaultmap)
 static GHashTable *
 return_colorprofile_configlist(GHashTable *config_rc,  gboolean init_values)
 {
+	init_prop_string(&config_rc, &main_v->props.tab_color_active, "tab_color_active:", NULL);
 	init_prop_string(&config_rc, &main_v->props.tab_color_modified, "tab_color_modified:", init_values ?"#0000FF":NULL);
 	init_prop_string(&config_rc, &main_v->props.tab_color_loading, "tab_color_loading:", init_values ?"#C7C7C7":NULL );
 	init_prop_string(&config_rc, &main_v->props.tab_color_error, "tab_color_error:", init_values ?"#FF0000":NULL);
@@ -1170,6 +1163,8 @@ return_colorprofile_configlist(GHashTable *config_rc,  gboolean init_values)
 	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_CURSOR], "cursor_color:", init_values ?"#000000":NULL);
 	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_SELECTION], "selection_color:", init_values ?"#000099":NULL);
 	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_CURSOR_HIGHLIGHT], "cursor_highlight_color:", init_values ?"#ffff33":NULL);
+	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_MARGIN_FG], "margin_fg:", init_values ?"#000000":NULL);
+	init_prop_string(&config_rc, &main_v->props.btv_color_str[BTV_COLOR_MARGIN_BG], "margin_bg:", init_values ?"#dddddd":NULL);
 	init_prop_arraylist(&config_rc, &main_v->props.textstyles, "textstyles3:", 6, init_values);
 	return config_rc;
 }
@@ -1189,6 +1184,7 @@ rcfile_parse_colorprofile(GFile * file)
 	}
 	retval = parse_config_file(configlist, file);
 	free_configlist(configlist);
+
 	/*setup_colors_after_parse(project->session);*/
 	return retval;
 }
@@ -1216,6 +1212,11 @@ return_globalsession_configlist(gboolean init_values)
 	init_prop_integer(&config_rc, &main_v->globses.left_panel_width, "left_panel_width:", 250, init_values);
 	/*init_prop_integer   (&config_rc, &main_v->globses.lasttime_filetypes, "lasttime_filetypes:", 0, init_values);
 	   init_prop_integer   (&config_rc, &main_v->globses.lasttime_encodings, "lasttime_encodings:", 0, init_values); */
+	init_prop_integer(&config_rc, &main_v->globses.print_headers, "print_headers:", 1,
+					  init_values);
+	init_prop_integer(&config_rc, &main_v->globses.print_linenumbers, "print_linenumbers:", 1,
+					  init_values);
+	init_prop_string_with_escape(&config_rc, &main_v->globses.print_fontstring, "print_fontstring:", NULL);
 	init_prop_integer(&config_rc, &main_v->globses.bookmarks_default_store, "bookmarks_default_store:", 1,
 					  init_values);
 	init_prop_integer(&config_rc, &main_v->globses.image_thumbnail_refresh_quality,
@@ -1231,6 +1232,7 @@ return_globalsession_configlist(gboolean init_values)
 					  "<a href=\"%r\"><img src=\"%t\" width=\"%x\" height=\"%y\" border=\"0\"></a>" : NULL));
 	init_prop_integer(&config_rc, &main_v->globses.filter_on_selection_mode,
 					  "filter_on_selection_mode:", 0, init_values);
+	init_prop_string_with_escape(&config_rc, &main_v->globses.last_project_dir, "last_project_dir:", NULL);
 	init_prop_arraylist(&config_rc, &main_v->globses.filefilters, "filefilters:", 4, init_values);
 	init_prop_arraylist(&config_rc, &main_v->globses.reference_files, "reference_files:", 2, init_values);
 	init_prop_limitedstringlist(&config_rc, &main_v->globses.recent_projects, "recent_projects:",
@@ -1267,6 +1269,8 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_integer(&configlist, &session->snr3_replacetype, "snr_replacetype:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_scope, "snr_scope:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_casesens, "snr_casesens:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_showinoutputbox, "snr_showinoutputbox:", 1, FALSE);
+	init_prop_integer(&configlist, &session->snr3_ignorebackupfiles, "snr_ignorebackupfiles:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_escape_chars, "snr_escape_chars:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_dotmatchall, "snr_dotmatchall:", 1, FALSE);
 	init_prop_integer(&configlist, &session->snr3_recursion_level, "snr_recursion_level:", 5, FALSE);
@@ -1289,9 +1293,11 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_string(&configlist, &session->template, "template:", NULL);
 	init_prop_string_with_escape(&configlist, &session->webroot, "webroot:", NULL);
 	init_prop_string_with_escape(&configlist, &session->documentroot, "documentroot:", NULL);
+	init_prop_limitedstringlist(&configlist, &session->snr3_basedir_history, "snr_basedir_history:", 15, FALSE);
 	init_prop_limitedstringlist(&configlist, &session->searchlist, "searchlist:", 15, FALSE);
 	init_prop_limitedstringlist(&configlist, &session->replacelist, "replacelist:", 15, FALSE);
-	init_prop_limitedstringlist(&configlist, &session->filegloblist, "filegloblist:", 15, FALSE);
+	init_prop_limitedstringlist(&configlist, &session->filegloblist, "filegloblist:", 30, FALSE);
+	
 	init_prop_stringlist(&configlist, &session->classlist, "classlist:", FALSE);
 	init_prop_stringlist(&configlist, &session->colorlist, "colorlist:", FALSE);
 	init_prop_stringlist(&configlist, &session->targetlist, "targetlist:", FALSE);
