@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snr3.c - search and replace
  *
- * Copyright (C) 2011,2012,2013,2014 Olivier Sessink
+ * Copyright (C) 2011,2012,2013,2014,2017,2019 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -368,31 +368,41 @@ static Tsnr3result * sn3run_add_result(Tsnr3run *s3run, gulong so, gulong eo, gp
 	s3result->doc = doc;
 	g_queue_push_tail(&s3run->results, s3result);
 	if (s3run->showinoutputbox && doc && DOCUMENT(doc)->uri) {
-		GtkTextIter it1, it2, tmpit;
-		gchar *curi, *text;
+		GtkTextIter it1, it2, it3, it4, tmpit;
+		gchar *curi, *text, *prefix, *match, *suffix;
 		guint line;
 		curi = g_file_get_uri(DOCUMENT(doc)->uri);
-		gtk_text_buffer_get_iter_at_offset(DOCUMENT(doc)->buffer, &it1, so);
-		line = gtk_text_iter_get_line(&it1)+1;
-		gtk_text_buffer_get_iter_at_offset(DOCUMENT(doc)->buffer, &it2, eo);
+		gtk_text_buffer_get_iter_at_offset(DOCUMENT(doc)->buffer, &it2, so);
+		line = gtk_text_iter_get_line(&it2)+1;
+		gtk_text_buffer_get_iter_at_offset(DOCUMENT(doc)->buffer, &it3, eo);
+		it4 = it3;
+		it1 = it2;
+		/* it2 is at so, it3 at eo */
 		if (!gtk_text_iter_starts_line(&it1)) {
 			if (gtk_text_iter_get_line_offset(&it1) <= 40)
 				gtk_text_iter_set_line_offset(&it1, 0);
 			else
 				gtk_text_iter_backward_chars(&it1, 40);
 		}
-		if (!gtk_text_iter_ends_line(&it2)) {
-			tmpit = it2;
-			gtk_text_iter_forward_to_line_end(&it2);
-			if (gtk_text_iter_get_offset(&it2) > (gtk_text_iter_get_offset(&tmpit)+40)) {
-				it2 = tmpit;
-				gtk_text_iter_forward_chars(&it2, 40);
+		if (!gtk_text_iter_ends_line(&it4)) {
+			tmpit = it4;
+			gtk_text_iter_forward_to_line_end(&it4);
+			if (gtk_text_iter_get_offset(&it4) > (gtk_text_iter_get_offset(&tmpit)+40)) {
+				it4 = tmpit;
+				gtk_text_iter_forward_chars(&it4, 40);
 			}
 		}
-		text = gtk_text_buffer_get_text(DOCUMENT(doc)->buffer, &it1, &it2, TRUE);
-		outputbox_add_line(s3run->bfwin, curi, line, text);
+		/*text = gtk_text_buffer_get_text(DOCUMENT(doc)->buffer, &it1, &it2, TRUE);*/
+		prefix = gtk_text_buffer_get_text(DOCUMENT(doc)->buffer, &it1, &it2, TRUE);
+		match = gtk_text_buffer_get_text(DOCUMENT(doc)->buffer, &it2, &it3, TRUE);
+		suffix = gtk_text_buffer_get_text(DOCUMENT(doc)->buffer, &it3, &it4, TRUE);
+		text = g_strconcat(prefix?prefix:"", "<b>", match?match:"","</b>", suffix?suffix:"", NULL);
+		outputbox_add_line_markup(s3run->bfwin, curi, line, text);
 		g_free(curi);
 		g_free(text);
+		g_free(prefix);
+		g_free(suffix);
+		g_free(match);
 	}
 	return s3result;
 }
@@ -750,7 +760,7 @@ snr3_cancel_run(Tsnr3run *s3run) {
 }
 
 static void
-snr3run_resultcleanup(Tsnr3run *s3run)
+snr3run_resultcleanup(Tsnr3run *s3run, gboolean clear_outputbox)
 {
 	GList *tmplist;
 	for (tmplist=g_list_first(s3run->results.head);tmplist;tmplist=g_list_next(tmplist)) {
@@ -762,13 +772,15 @@ snr3run_resultcleanup(Tsnr3run *s3run)
 	s3run->curoffset=0;
 	s3run->resultnumdoc=0;
 	s3run->searchednumdoc=0;
+	if (clear_outputbox && s3run->showinoutputbox) {
+		outputbox_clear(s3run->bfwin);
+	}
 }
 
 /* called from bfwin.c for simplesearch */
 void
-snr3run_free(Tsnr3run *s3run, gboolean remove_highlights) {
+snr3run_free(Tsnr3run *s3run, gboolean remove_highlights, gboolean clear_outputbox) {
 	DEBUG_MSG("snr3run_free, started for %p\n",s3run);
-	snr3_cancel_run(s3run);
 	if (s3run->curbuf)
 		g_free(s3run->curbuf);
 	bfwin_current_document_change_remove_by_data(s3run->bfwin, s3run);
@@ -792,7 +804,7 @@ snr3run_free(Tsnr3run *s3run, gboolean remove_highlights) {
 		remove_all_highlights_in_doc(s3run->bfwin->current_document);
 	}
 	DEBUG_MSG("snr3run_free, resultcleanup\n");
-	snr3run_resultcleanup(s3run);
+	snr3run_resultcleanup(s3run, clear_outputbox);
 	g_slice_free(Tsnr3run, s3run);
 }
 static gboolean compile_regex(Tsnr3run *s3run) {
@@ -884,6 +896,7 @@ void snr3run_bookmark_all(Tsnr3run *s3run) {
 static void
 threaded_all_ready(void *data) {
 	Tsnr3run *s3run=data;
+	DEBUG_MSG("threaded_all_ready, s3run=%p\n",s3run);
 	s3run->unre_action_id = 0;
 	s3run->curdoc = NULL;
 	if (s3run->dialog) {
@@ -901,6 +914,9 @@ threaded_all_ready(void *data) {
 	}
 	s3run->replaceall=FALSE;
 	s3run->findall=FALSE;
+	if (g_atomic_int_get(&s3run->cancelled)==1 && s3run->dialog==NULL) {
+		snr3run_free(s3run,TRUE,FALSE);
+	} 
 }
 
 static void
@@ -920,7 +936,7 @@ snr3_curdocchanged_cb(Tbfwin *bfwin, Tdocument *olddoc, Tdocument *newdoc, gpoin
 		if (s3run->dialog) {
 			highlight_run_in_doc(s3run, newdoc);
 		} else { /* simple search */
-			snr3run_resultcleanup(s3run);
+			snr3run_resultcleanup(s3run, TRUE);
 			snr3_run(s3run, NULL, newdoc, highlight_simple_search);
 		}
 	}
@@ -1084,7 +1100,7 @@ void snr3run_unrun(Tsnr3run *s3run) {
 	DEBUG_MSG("snr3run_unrun, before decrement runcount=%d\n",s3run->runcount);
 	if (g_atomic_int_dec_and_test(&s3run->runcount)) {
 		s3run->callback(s3run);
-		DEBUG_MSG("runcount 0, after the callback\n");
+		DEBUG_MSG("runcount 0, after the callback (possibly s3run has been free'ed now)\n");
 	}
 }
 
@@ -1092,7 +1108,7 @@ static void
 snr3run_multiset(Tsnr3run *s3run,
 				const gchar *query, const gchar *replace,
 				Tsnr3type type, Tsnr3replace replacetype,
-				Tsnr3scope scope) {
+				Tsnr3scope scope, gboolean showinoutputbox) {
 	g_free(s3run->query);
 	s3run->query = g_strdup(query);
 	g_free(s3run->replace);
@@ -1101,7 +1117,8 @@ snr3run_multiset(Tsnr3run *s3run,
 	s3run->type = type;
 	s3run->replacetype = replacetype;
 	s3run->scope = scope;
-	snr3run_resultcleanup(s3run);
+	s3run->showinoutputbox = showinoutputbox;
+	snr3run_resultcleanup(s3run, TRUE);
 	g_queue_init(&s3run->results);
 
 }
@@ -1148,7 +1165,7 @@ gpointer simple_search_run(Tbfwin *bfwin, const gchar *string, Tsnr3type type
 	Tsnr3run *s3run;
 
 	s3run = snr3run_new(bfwin, NULL);
-	snr3run_multiset(s3run, string, NULL, type,snr3replace_string,snr3scope_doc);
+	snr3run_multiset(s3run, string, NULL, type,snr3replace_string,snr3scope_doc, FALSE);
 	simple_search_add_to_history(bfwin, string);
 	s3run->dotmatchall = dotmatchall;
 	s3run->is_case_sens = casesens;
@@ -1204,9 +1221,10 @@ static gint
 snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 {
 	const gchar *query, *replace;
-	gint type, replacetype, scope, dotmatchall, escapechars, recursion_level;
+	gint type, replacetype, scope, dotmatchall, escapechars, recursion_level, showinoutputbox, ignorebackupfiles;
 	gboolean is_case_sens;
 	gint retval=0;
+	gchar *cbasedir;
 	GFile *basedir;
 	const gchar *filepattern;
 	DEBUG_MSG("s3run->in_replace=%d\n",s3run->in_replace);
@@ -1220,10 +1238,13 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 	scope = gtk_combo_box_get_active(GTK_COMBO_BOX(snrwin->scope));
 	is_case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->matchCase));
 	dotmatchall = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->dotmatchall));
+	showinoutputbox = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->showinoutputbox));
+	ignorebackupfiles = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->ignorebackupfiles));
 	escapechars = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(snrwin->escapeChars));
 	recursion_level = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(snrwin->recursion_level));
 
-	basedir = g_file_new_for_commandline_arg(gtk_entry_get_text(GTK_ENTRY(snrwin->basedir)));
+	cbasedir = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(snrwin->basedir))));
+	basedir = g_file_new_for_commandline_arg(cbasedir);
 	filepattern = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(snrwin->filepattern))));
 
 	if (is_case_sens != s3run->is_case_sens) {
@@ -1254,6 +1275,19 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 		s3run->type = type;
 		retval |= 1;
 	}
+	if (showinoutputbox != s3run->showinoutputbox) {
+		snr3_cancel_run(s3run);
+		if (!showinoutputbox) {
+			outputbox_cleanup(snrwin->bfwin);
+		}
+		s3run->showinoutputbox = showinoutputbox;
+		retval |= 1;
+	}
+	if (ignorebackupfiles != s3run->ignorebackupfiles) {
+		snr3_cancel_run(s3run);
+		s3run->ignorebackupfiles = ignorebackupfiles;
+		retval |= 1;
+	}
 	if (escapechars != s3run->escape_chars) {
 		snr3_cancel_run(s3run);
 		s3run->escape_chars = escapechars;
@@ -1304,7 +1338,7 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 
 	if ((retval & 1) != 0) {
 		remove_all_highlights_in_doc(snrwin->bfwin->current_document);
-		snr3run_resultcleanup(s3run);
+		snr3run_resultcleanup(s3run, TRUE);
 	}
 	/*gtk_widget_hide(snrwin->searchfeedback);*/
 
@@ -1315,6 +1349,8 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 		snrwin->bfwin->session->snr3_scope = scope;
 		snrwin->bfwin->session->snr3_casesens = is_case_sens;
 		snrwin->bfwin->session->snr3_escape_chars = escapechars;
+		snrwin->bfwin->session->snr3_showinoutputbox = showinoutputbox;
+		snrwin->bfwin->session->snr3_ignorebackupfiles = ignorebackupfiles;
 		snrwin->bfwin->session->snr3_dotmatchall = dotmatchall;
 		snrwin->bfwin->session->snr3_recursion_level = recursion_level;
 		if (scope == snr3scope_files && s3run->basedir) {
@@ -1322,7 +1358,14 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 			snrwin->bfwin->session->snr3_filepattern = g_strdup(s3run->filepattern);
 			g_free(snrwin->bfwin->session->snr3_basedir);
 			snrwin->bfwin->session->snr3_basedir = g_file_get_uri(s3run->basedir);
+			if (s3run->filepattern && s3run->filepattern[0] != '\0')
+				snrwin->bfwin->session->filegloblist = add_to_history_stringlist(snrwin->bfwin->session->filegloblist, s3run->filepattern,TRUE);
+			if (cbasedir && cbasedir[0] != '\0')
+				snrwin->bfwin->session->snr3_basedir_history = add_to_history_stringlist(snrwin->bfwin->session->snr3_basedir_history, cbasedir,TRUE);
 		}
+		snrwin->bfwin->session->searchlist = add_to_history_stringlist(snrwin->bfwin->session->searchlist, s3run->query, TRUE);
+		if (s3run->replace && s3run->replace[0] != '\0')
+			snrwin->bfwin->session->replacelist = add_to_history_stringlist(snrwin->bfwin->session->replacelist, s3run->replace,TRUE);
 	}
 	DEBUG_MSG("snr3run_init_from_gui, return %d\n", retval);
 	return retval;
@@ -1368,7 +1411,7 @@ snr3_advanced_response(GtkDialog * dialog, gint response, TSNRWin * snrwin)
 							&& snrwin->s3run->results.head
 							&& S3RESULT(snrwin->s3run->results.head->data)->doc != snrwin->bfwin->current_document) {
 		DEBUG_MSG("restart the search on the new active document\n");
-		snr3run_resultcleanup(snrwin->s3run);
+		snr3run_resultcleanup(snrwin->s3run, TRUE);
 		snr3_run(snrwin->s3run, snrwin, snrwin->s3run->bfwin->current_document, dialog_changed_run_ready_cb);
 		return;
 	}
@@ -1382,12 +1425,6 @@ snr3_advanced_response(GtkDialog * dialog, gint response, TSNRWin * snrwin)
 	if (guichange == -1)
 		return;
 
-	snrwin->bfwin->session->searchlist = add_to_history_stringlist(snrwin->bfwin->session->searchlist, s3run->query, TRUE);
-	if (s3run->replace && s3run->replace[0] != '\0')
-		snrwin->bfwin->session->replacelist = add_to_history_stringlist(snrwin->bfwin->session->replacelist, s3run->replace,TRUE);
-	if (s3run->scope == snr3scope_files && s3run->filepattern && s3run->filepattern[0] != '\0')
-		snrwin->bfwin->session->filegloblist = add_to_history_stringlist(snrwin->bfwin->session->filegloblist, s3run->filepattern,TRUE);
-
 	switch(response) {
 		case SNR_RESPONSE_FIND:
 			if ((guichange & 1) != 0) {
@@ -1426,7 +1463,7 @@ snr3_advanced_response(GtkDialog * dialog, gint response, TSNRWin * snrwin)
 		case SNR_RESPONSE_REPLACE_ALL:
 			snr3_cancel_run(s3run);
 			s3run->replaceall=TRUE;
-			snr3run_resultcleanup(s3run);
+			snr3run_resultcleanup(s3run, TRUE);
 			s3run->unre_action_id = new_unre_action_id();
 
 			replace_all_buttons(s3run, FALSE);
@@ -1435,7 +1472,7 @@ snr3_advanced_response(GtkDialog * dialog, gint response, TSNRWin * snrwin)
 		break;
 		case SNR_RESPONSE_FIND_ALL:
 			snr3_cancel_run(s3run);
-			snr3run_resultcleanup(s3run);
+			snr3run_resultcleanup(s3run, TRUE);
 			replace_all_buttons(s3run, FALSE);
 			s3run->findall=TRUE;
 			snr3_run(s3run, snrwin, s3run->bfwin->current_document, threaded_all_ready);
@@ -1466,6 +1503,8 @@ static void snr_dialog_show_widgets(TSNRWin * snrwin) {
 	widget_set_visible(snrwin->replaceType, (searchtype == snr3type_pcre));
 	widget_set_visible(snrwin->replaceTypeL, (searchtype == snr3type_pcre));
 	widget_set_visible(snrwin->replace, (searchtype != snr3type_pcre || replacetype == snr3replace_string));
+	widget_set_visible(snrwin->showinoutputbox, (scope != snr3scope_files));
+	widget_set_visible(snrwin->ignorebackupfiles, (scope == snr3scope_files));
 	widget_set_visible(snrwin->escapeChars, (searchtype == snr3type_string));
 	widget_set_visible(snrwin->dotmatchall, (searchtype == snr3type_pcre));
 
@@ -1500,15 +1539,21 @@ snr3win_destroy_cb(GtkWidget *widget, gpointer user_data)
 {
 	TSNRWin *snrwin=user_data;
 	DEBUG_MSG("snr3win_destroy_cb, user_data=%p\n",user_data);
-	snr3run_free(snrwin->s3run, TRUE);
+	snr3_cancel_run(snrwin->s3run);
+	snrwin->s3run->dialog = NULL;
+	if (g_atomic_int_get(&snrwin->s3run->runcount)==0) {
+		snr3run_free(snrwin->s3run, TRUE, FALSE);
+	}
 	g_slice_free(TSNRWin, snrwin);
 }
 
 static void
-snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s3scope)
+snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s3scope, const gchar *files_basedir)
 {
 	TSNRWin *snrwin;
 	GtkWidget *table, *vbox;
+	gchar *set_basedir=NULL;
+	gboolean free_set_basedir=FALSE;
 	gint currentrow=0;
 	/*GtkTextIter start, end; */
 	unsigned int i = 0;
@@ -1548,7 +1593,7 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 	vbox = gtk_dialog_get_content_area(GTK_DIALOG(snrwin->dialog));
 
 	table =
-		dialog_table_in_vbox(3, 4, 6/*borderwidth*/, vbox, TRUE,
+		dialog_table_in_vbox(3, 7, 6/*borderwidth*/, vbox, TRUE,
 							 TRUE, 0);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	currentrow=0;
@@ -1639,27 +1684,47 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 
 	currentrow++;
 
-	snrwin->basedir = gtk_entry_new();
-	snrwin->basedirL = dialog_mnemonic_label_in_table(_("Basedir: "), snrwin->basedir, table, 0, 1, currentrow, currentrow+1);
-	if (bfwin->session->snr3_basedir && bfwin->session->snr3_basedir[0]!='\0') {
-		gtk_entry_set_text(GTK_ENTRY(snrwin->basedir), bfwin->session->snr3_basedir);
+	if (files_basedir!=NULL) {
+		set_basedir = files_basedir;
+	} else if (bfwin->session->snr3_basedir && bfwin->session->snr3_basedir[0]!='\0') {
+		set_basedir = bfwin->session->snr3_basedir;
 	} else if (bfwin->current_document && bfwin->current_document->uri) {
 		GFile *parent = g_file_get_parent(bfwin->current_document->uri);
 		gchar *tmp = g_file_get_uri(parent);
-		gtk_entry_set_text(GTK_ENTRY(snrwin->basedir), tmp);
+		set_basedir = g_file_get_uri(parent);
+		free_set_basedir=TRUE;
 		g_object_unref(parent);
-		g_free(tmp);
 	} else if (bfwin->session->recent_dirs) {
 		GList *tmplist = g_list_last(bfwin->session->recent_dirs);
 		if (tmplist && tmplist->data)
-			gtk_entry_set_text(GTK_ENTRY(snrwin->basedir), tmplist->data);
+			set_basedir = tmplist->data;
 	}
+	snrwin->basedir = combobox_with_popdown(set_basedir, bfwin->session->snr3_basedir_history, TRUE);
+	if (free_set_basedir) {
+		g_free(set_basedir);
+	}
+	snrwin->basedirL = dialog_mnemonic_label_in_table(_("Basedir: "), snrwin->basedir, table, 0, 1, currentrow, currentrow+1);
+	
 	gtk_table_attach(GTK_TABLE(table), snrwin->basedir, 1, 3, currentrow, currentrow+1,GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
-	snrwin->basedirB = file_but_new2(snrwin->basedir, 1, bfwin, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+	
+	/* The entry itself can be accessed by calling gtk_bin_get_child() on the combo box. */
+	snrwin->basedirB = file_but_new2(gtk_bin_get_child(GTK_BIN(snrwin->basedir)), 1, bfwin, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 	gtk_table_attach(GTK_TABLE(table), snrwin->basedirB, 3, 4, currentrow, currentrow+1,GTK_FILL, GTK_SHRINK, 0, 0);
 
 	currentrow++;
 
+	snrwin->showinoutputbox = dialog_check_button_in_table(_("Show results in Output Pane"), bfwin->session->snr3_showinoutputbox, table,
+										0, 4, currentrow, currentrow+1);
+	gtk_widget_set_tooltip_text(snrwin->showinoutputbox, _("Show all results in the Output Pane on the bottom of the document list"));
+
+	currentrow++;
+
+	snrwin->ignorebackupfiles = dialog_check_button_in_table(_("Ignore backup files"), bfwin->session->snr3_ignorebackupfiles, table,
+										0, 4, currentrow, currentrow+1);
+	gtk_widget_set_tooltip_text(snrwin->ignorebackupfiles, _("Ignore backup files *~ created by Bluefish"));
+
+	currentrow++;
+
 	snrwin->matchCase = dialog_check_button_in_table(_("Case sensitive _matching"), bfwin->session->snr3_casesens, table,
 										0, 4, currentrow, currentrow+1);
 	gtk_widget_set_tooltip_text(snrwin->matchCase, _("Only match if case (upper/lower) is identical."));
@@ -1736,8 +1801,9 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 	g_signal_connect(snrwin->scope, "changed", G_CALLBACK(snr_combobox_changed), snrwin);
 	g_signal_connect(snrwin->matchCase, "toggled", G_CALLBACK(snr_option_toggled), snrwin);
 	g_signal_connect(snrwin->escapeChars, "toggled", G_CALLBACK(snr_option_toggled), snrwin);
-	g_signal_connect(snrwin->escapeChars, "toggled", G_CALLBACK(snr_option_toggled), snrwin);
-
+	g_signal_connect(snrwin->showinoutputbox, "toggled", G_CALLBACK(snr_option_toggled), snrwin);
+	g_signal_connect(snrwin->ignorebackupfiles, "toggled", G_CALLBACK(snr_option_toggled), snrwin);
+	g_signal_connect(snrwin->dotmatchall, "toggled", G_CALLBACK(snr_option_toggled), snrwin);
 
 	snr_dialog_show_widgets(snrwin);
 
@@ -1764,16 +1830,35 @@ snr3_advanced_dialog(Tbfwin * bfwin, const gchar *findtext)
 		/* check if it is a multiline selection */
 		if (gtk_text_iter_get_line(&so)==gtk_text_iter_get_line(&eo)) {
 			gchar *tmp = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(doc->buffer), &so, &eo, TRUE);
-			snr3_advanced_dialog_backend(bfwin, findtext?findtext:tmp, bfwin->session->snr3_scope);
+			snr3_advanced_dialog_backend(bfwin, findtext?findtext:tmp, bfwin->session->snr3_scope, NULL);
 			g_free(tmp);
 		} else {
-			snr3_advanced_dialog_backend(bfwin, findtext?findtext:"", snr3scope_selection);
+			snr3_advanced_dialog_backend(bfwin, findtext?findtext:"", snr3scope_selection, NULL);
 		}
 	} else {
-		snr3_advanced_dialog_backend(bfwin, findtext?findtext:"", bfwin->session->snr3_scope);
+		snr3_advanced_dialog_backend(bfwin, findtext?findtext:"", bfwin->session->snr3_scope, NULL);
 	}
 }
 
+void
+snr3_advanced_dialog_files(Tbfwin * bfwin, const gchar *curi)
+{
+	gchar *searchstring=NULL;
+	Tdocument *doc = bfwin->current_document;
+	if (doc) {
+		GtkTextIter so, eo;
+		if (gtk_text_buffer_get_selection_bounds(GTK_TEXT_BUFFER(doc->buffer),&so,&eo) ) {
+			if (gtk_text_iter_get_line(&so)==gtk_text_iter_get_line(&eo)) {
+				searchstring = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(doc->buffer), &so, &eo, TRUE);
+			}
+		}
+	}
+	snr3_advanced_dialog_backend(bfwin, searchstring, snr3scope_files, curi);
+	if (searchstring)
+		g_free(searchstring);
+} 
+
+
 static void
 extern_doc_backend(Tsnr3run * s3run, Tdocument *doc, guint so, guint eo)
 {
@@ -1801,7 +1886,7 @@ snr3_run_extern_replace(Tdocument * doc, const gchar * search_pattern, Tsnr3scop
 	gint so,eo;
 	GList *tmplist;
 	Tsnr3run * s3run = snr3run_new(doc->bfwin, NULL);
-	snr3run_multiset(s3run, search_pattern, NULL, type,snr3replace_string,scope);
+	snr3run_multiset(s3run, search_pattern, NULL, type,snr3replace_string,scope,FALSE);
 	s3run->dotmatchall = dotmatchall;
 	s3run->replace = g_strdup(replace_pattern);
 	update_snr3run(s3run);
@@ -1836,6 +1921,6 @@ snr3_run_extern_replace(Tdocument * doc, const gchar * search_pattern, Tsnr3scop
 			g_warning("snr3_run_extern_replace does not support replace in files\n");
 		break;
 	}
-	snr3run_free(s3run, FALSE); /* We do not need to remove hightlights in current_document, so we pass FALSE */
+	snr3run_free(s3run, FALSE, FALSE); /* We do not need to remove hightlights in current_document, so we pass FALSE */
 }
 
