@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snr3.h - search and replace
  *
- * Copyright (C) 2011 Olivier Sessink
+ * Copyright (C) 2011-2017 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,7 @@ typedef struct {
 	gboolean select_matches;
 	gboolean bookmark_matches;
 	gboolean showinoutputbox;
+	gboolean ignorebackupfiles;
 
 	/* the resultss of a search run */
 	gboolean in_replace; /* TRUE if the code is in a replace, so the doc_insert and doc_delete signals do not need to do anything */
@@ -148,6 +149,8 @@ typedef struct {
 	GtkWidget *replaceTypeL;
 	GtkWidget *overlappingMatches;
 	GtkWidget *matchCase;
+	GtkWidget *showinoutputbox;
+	GtkWidget *ignorebackupfiles;
 	GtkWidget *escapeChars;
 	GtkWidget *dotmatchall;
 	GtkWidget *select_match;
@@ -172,12 +175,13 @@ typedef struct {
 void snr3_run(Tsnr3run *s3run, TSNRWin *snrwin, Tdocument *doc, void (*callback)(void *)); /* called from snr3_files.c */
 void snr3_run_in_doc(Tsnr3run *s3run, Tdocument *doc, gint so, gint eo, gboolean update);
 void snr3_run_go(Tsnr3run *s3run, gboolean forward);
-void snr3run_free(Tsnr3run *s3run, gboolean remove_highlights);
+void snr3run_free(Tsnr3run *s3run, gboolean remove_highlights, gboolean clear_outputbox);
 void snr3run_unrun(Tsnr3run *s3run);
 gpointer simple_search_run(Tbfwin *bfwin, const gchar *string, Tsnr3type type
 		, gboolean casesens, gboolean dotmatchall, gboolean unescape);
 void simple_search_next(Tbfwin *bfwin);
 void snr3_advanced_dialog(Tbfwin * bfwin, const gchar *searchstring);
+void snr3_advanced_dialog_files(Tbfwin * bfwin, const gchar *curi);
 
 void snr3_run_extern_replace(Tdocument * doc, const gchar * search_pattern, Tsnr3scope scope,
 							 Tsnr3type type, gboolean is_case_sens, const gchar * replace_pattern,
